package com.cirs.biz.service;

import java.util.List;
import com.cirs.biz.domain.FactoryScore;

/**
 * 厂家得分Service接口
 * 
 * @author allen
 * @date 2023-12-10
 */
public interface IFactoryScoreService 
{
    /**
     * 查询厂家得分
     * 
     * @param id 厂家得分主键
     * @return 厂家得分
     */
    public FactoryScore selectFactoryScoreById(Long id);

    /**
     * 查询厂家得分列表
     * 
     * @param factoryScore 厂家得分
     * @return 厂家得分集合
     */
    public List<FactoryScore> selectFactoryScoreList(FactoryScore factoryScore);

    /**
     * 新增厂家得分
     * 
     * @param factoryScore 厂家得分
     * @return 结果
     */
    public int insertFactoryScore(FactoryScore factoryScore);

    /**
     * 修改厂家得分
     * 
     * @param factoryScore 厂家得分
     * @return 结果
     */
    public int updateFactoryScore(FactoryScore factoryScore);

    /**
     * 批量删除厂家得分
     * 
     * @param ids 需要删除的厂家得分主键集合
     * @return 结果
     */
    public int deleteFactoryScoreByIds(Long[] ids);

    /**
     * 删除厂家得分信息
     * 
     * @param id 厂家得分主键
     * @return 结果
     */
    public int deleteFactoryScoreById(Long id);

    public String importTrain(List<FactoryScore> scoreList, boolean updateSupport, String operName);
}
